/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.control;

import ikr.simlib.control.notification.MultiNotificationHandler;
import ikr.simlib.control.notification.NotificationHandler;
import ikr.simlib.control.simulationcontrol.callbacks.SimulationControlManager;

/**
 * Simulation controls all simulation phases. You can register notifiers to
 * control each phase of the simulation.
 */
public abstract class Simulation {

   /**
    * the handler of the end of the transient phase
    */
   protected MultiNotificationHandler transientPhaseHandler;

   /**
    * the handler of the end of a batch
    */
   protected MultiNotificationHandler batchHandler;

   public Simulation() {
      transientPhaseHandler = new MultiNotificationHandler(new NotificationHandler() {
         @Override
         public void handleNotification() {
            handleEndOfTransientPhase();
         }
      });

      batchHandler = new MultiNotificationHandler(new NotificationHandler() {
         @Override
         public void handleNotification() {
            handleEndOfCurrentBatch();
         }
      });
   }

   public void run() {
      initSimulation();
      startSimulation();
      runSimulation();
      printResults();
   }

   public void initSimulation() {
      SimulationControlManager.getInstance().initSimulation();
   }

   public void startSimulation() {
      SimulationControlManager.getInstance().startSimulation(this);
   }

   public abstract void runSimulation();

   public abstract void printResults();

   public MultiNotificationHandler getTransientPhaseNotifier() {
      return transientPhaseHandler;
   }

   public MultiNotificationHandler getBatchNotifier() {
      return batchHandler;
   }

   public abstract void handleEndOfTransientPhase();

   public abstract void handleEndOfCurrentBatch();
}
