/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

/** 
 * <h1>Simulation Control</h1> <br>
 * 
 * The sequence control is a summary of tasks, that take place in every
 * simulation, which can be adapted to each problem in a flexible manner. It can
 * also be applied to simple problems without change and without exact knowledge
 * of the internal structures.<br>
 * In order to control a sequence the abstract base class
 * {@link ikr.simlib.control.Simulation} has been developed that defines
 * functions for the individual steps, which are executed during a simulation.
 * Most of these functions are empty and can be overridden in derived classes.<br>
 * <br>
 * The class {@link ikr.simlib.control.StdSimulation} derived from
 * {@link ikr.simlib.control.Simulation} has been pre-defined and will suffice
 * for many simulations. One entity of this class is generally declared in the
 * <code>main()</code> function of the simulation program and requires the
 * following parameters upon constructor call:
 * <ul>
 * <li>a reference to the model (class Model, {@link ikr.simlib.model}) which is
 * to be used for simulation<br>
 * <li>a reference to the simulation environment (class
 * {@link ikr.simlib.control.SimulationEnvironment})
 * </ul>
 * <h2>Simulation Phases</h2> The individual phases of a simulation - realized
 * by methods of the class {@link ikr.simlib.control.Simulation} - are processed
 * when the method <code>run()</code> from {@link ikr.simlib.control.Simulation}
 * has been called. Normally, this occurs in the <code>main()</code> function of
 * the simulation program.<br>
 * <br>
 * In the derived class {@link ikr.simlib.control.StdSimulation}, the methods
 * have been (partly) overridden so that they mirror a normal sequence of a
 * simulation.<br>
 * <br>
 * Besides this approach, there is another possibility to achieve simulation
 * results, which makes use of batch parallelization. Here the intermediate
 * results of each batch are written to a special file. In a second run, all
 * these batch results are read in and the final results are calculated. Each
 * parallelized run goes through all of the above mentioned phases. Thus there
 * is a transient phase for each run, which means that in total more has to be
 * simulated in comparison to a nonparallelized simulation. As consequence the
 * usage of batch parallelization is only reasonable when having more CPUs or
 * cores than simulations or when the time to execute the simulation needs to be
 * reduced.
 */
package ikr.simlib.control;