/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.control.simulationcontrol;

import ikr.simlib.control.Simulation;
import ikr.simlib.control.simulationcontrol.callbacks.InitSimulationCallbackI;
import ikr.simlib.control.simulationcontrol.callbacks.SimulationControlManager;
import ikr.simlib.control.simulationcontrol.callbacks.StartBatchCallbackI;
import ikr.simlib.control.simulationcontrol.callbacks.StartSimulationCallbackI;
import ikr.simlib.control.simulationcontrol.callbacks.StartTransientPhaseCallbackI;
import ikr.simlib.control.simulationcontrol.callbacks.StopBatchCallbackI;
import ikr.simlib.control.simulationcontrol.callbacks.StopSimulationCallbackI;
import ikr.simlib.control.simulationcontrol.callbacks.StopTransientPhaseCallbackI;

import java.util.logging.Logger;

/**
 * SimControlTracer outputs all phase changes to a given writer. Nice for
 * debugging.
 */
public class SimControlTracer {

   private static final Logger LOG = Logger.getLogger(SimControlTracer.class.getName());

   public SimControlTracer() {

      SimulationControlManager.getInstance().registerInitSimulationCallback(new InitSimulationCallbackI() {
         @Override
         public void handleInitSimulation() {
            LOG.info("### InitSimulation ###");
         }
      });

      SimulationControlManager.getInstance().registerStartBatchCallback(new StartBatchCallbackI() {
         @Override
         public void handleStartBatch(int batchNumber) {
            LOG.info("### StartBatch " + batchNumber + " ###");
         }
      });

      SimulationControlManager.getInstance().registerStartSimulationCallback(new StartSimulationCallbackI() {
         @Override
         public void handleStartSimulation(Simulation sim) {
            LOG.info("### StartSimulation ###");
         }
      });

      SimulationControlManager.getInstance().registerStartTransientPhaseCallback(new StartTransientPhaseCallbackI() {
         @Override
         public void handleStartTransientPhase() {
            LOG.info("### StartTransientPhase ###");
         }
      });

      SimulationControlManager.getInstance().registerStopBatchCallback(new StopBatchCallbackI() {
         @Override
         public void handleStopBatch(int batchNumber) {
            LOG.info("### StopBatch " + batchNumber + " ###");
         }
      });

      SimulationControlManager.getInstance().registerStopSimulationCallback(new StopSimulationCallbackI() {
         @Override
         public void handleStopSimulation() {
            LOG.info("### StopSimulation ###");
         }
      });

      SimulationControlManager.getInstance().registerStopTransientPhaseCallback(new StopTransientPhaseCallbackI() {
         @Override
         public void handleStopTransientPhase() {
            LOG.info("### StopTransientPhase ###");
         }
      });
   }
}
