/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

/** 
 * In order to reproduce the stochastic processes in the system (e.g., arrival
 * processes, service processes) using events it is necessary to generate random
 * variables for time stamps according to a pre-defined distribution function,
 * possibly taking the system state into consideration. The simulation library
 * offers a number of distributions. <br>
 * <h2>Generating Random Numbers</h2> The generation of random variables from a
 * distribution function is based on the generation of pseudo-random number
 * sequences with a large period following a uniform distribution between 0 and
 * 1. <br>
 * <br>
 * Some classes have been derived from
 * {@link ikr.simlib.random.RandomNumberGenerator} that serve as random number
 * generators. In most cases they override the index operator <code>
   operator()</code>
 * and return a random number between 0 and 1: <br>
 * {@link ikr.simlib.random.StdRandomNumberGenerator} is a relatively simple but
 * generally sufficient generator [1]. A global instance <code>
   SYSTEM_RNG</code> of
 * this type is generated that is used as the default random number generator
 * for the system. <br>
 * <br>
 * Random variables needed for the simulation are generated based on these
 * random numbers and a specific distribution function (e.g., for random service
 * times or arrival intervals). The distribution function must be inverted (Fig.
 * 1), which in some cases can be done exactly, in other cases only
 * approximately [2]. The task of inversion is executed by the distribution
 * classes. <br>
 * <br><center>
 * <img src="{@docRoot}/ikr/simlib/doc-files/rnGenerationP16.svg" width="600" alt="Creating a random number according to a certain distribution"> <br>
 * Creating a random number according to a certain distribution </center></td> <br>
 * <h2>Structure and Interface of Distribution Classes</h2> All distribution
 * classes are derived from the class
 * {@link ikr.simlib.distributions.Distribution}, which has the following
 * characteristics: <br>
 * a final reference <code>
   rng</code> to the its random number generator, which will
 * by default be initialized with a reference to the system default random
 * number generator (see {@link ikr.simlib.random}) <br>
 * <br>
 * Basically, discrete and continuous distributions are treated differently.
 * They are represented by the classes
 * {@link ikr.simlib.distributions.discrete.DiscreteDistribution} and
 * {@link ikr.simlib.distributions.continuous.ContinuousDistribution} , which
 * have been derived from {@link ikr.simlib.distributions.Distribution}. <br>
 * <br>
 * <br>
 * [1] S. K. PARK, K. W. MILLER:
 * "Random number generators: good ones are hard to find." Communications of the
 * ACM, pp. 1192-1201, Oct. 1988. <br>
 * [2] A. M. LAW, W. D. KELTON: Simulation Modeling & Analysis, 2nd edition,
 * McGraw- Hill, 1991
 */
package ikr.simlib.distributions;