/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.entities.multiphase;

import java.util.ArrayDeque;
import java.util.Deque;

public class ServerPriorityQueue {

   protected final int priority;
   protected ServerJob interruptedJob;
   protected final Deque<ServerJob> baseQueue;

   public ServerPriorityQueue(int priority) {
      this.priority = priority;
      baseQueue = new ArrayDeque<ServerJob>();
   }

   public void append(ServerJob job) {
      baseQueue.addLast(job);
   }

   public void insert(ServerJob job) {
      baseQueue.add(job);
   }

   public void putBack(ServerJob job) {
      interruptedJob = job;
   }

   public ServerJob popHead() {
      if (interruptedJob != null) {
         ServerJob job = interruptedJob;
         interruptedJob = null;
         return job;
      } else {
         return baseQueue.removeFirst();
      }
   }

   public ServerJob head() {
      if (interruptedJob != null) {
         return interruptedJob;
      } else {
         return baseQueue.peekFirst();
      }
   }

   public int getLength() {
      int size = baseQueue.size();
      if (interruptedJob != null) {
         size++;
      }
      return size;
   }

   public boolean isEmpty() {
      return interruptedJob == null && baseQueue.isEmpty();
   }

   public int getPriority() {
      return priority;
   }
}
