/*
 * Copyright (C) 2007-2014 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

/**
 * <h1>Event Handling</h1>
 * <p>
 * The simulation library is based on the principle of event-driven simulation.
 * Stemming from a model in which state transitions take place at discrete
 * points in time (therefore discrete time simulation), the simulated time of
 * the original system is represented by the real-time of the simulation so that
 * the "idle time" between state transitions is bridged and no further
 * processing time is needed (see Figure).<br>
 * </p>
 * 
 * <pre>
 * <center>
 * <img src="{@docRoot}/ikr/simlib/doc-files/IKRSimLib_Events.svg" width="600" 
 * alt="Events during the simulation"><br>
 * Events during the simulation</center>
 * </pre>
 * 
 * <p>
 * The state transitions are described with the help of events. These events
 * have two characteristics. One is a time stamp, which defines the simulated
 * point in time at which the event occurred. The other is a pointer to a
 * function which defines the actions of the event (e.g.the state transition).
 * An example of an event might be the arrival of a request at the generator or
 * the end of service for a request in a service unit.
 * </p>
 * <p>
 * In the current version of the IKR SimLib, there is no separate class which
 * models an event. Instead, events are created implicitly in the calendar upon
 * call of the
 * {@link ikr.simlib.events.calendar.Calendar#postEvent(CalendarCallback, CalendarCallback, PointInTime)}
 * method. When this method is called, the calendar schedules the execution of
 * the specified {@link ikr.simlib.events.CalendarCallback} upon the specified
 * system time. In addition, the calendar returns an
 * {@link ikr.simlib.events.EventToken}, which allows for event-specific
 * interaction with the calendar.
 * </p>
 */
package ikr.simlib.events;