/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.messages;

/**
 * IPPacket is specially designed for IP QoS simulations where fields like
 * source and destination as well as further fields to distinguish between
 * messages are needed
 */
public class IPPacket extends Message {

   private int sourceId;
   private int destinationId;
   private int groupId;
   private int typeId;

   private double tag;

   public IPPacket(long length, int sourceId, int destId, int group, int type) {
      super(length);
      this.sourceId = sourceId;
      destinationId = destId;
      groupId = group;
      typeId = type;
      tag = -1;
   }

   public int getSourceId() {
      return sourceId;
   }

   public void setSourceId(int sourceId) {
      this.sourceId = sourceId;
   }

   public int getDestinationId() {
      return destinationId;
   }

   public void setDestinationId(int destinationId) {
      this.destinationId = destinationId;
   }

   public int getGroupId() {
      return groupId;
   }

   public void setGroupId(int groupId) {
      this.groupId = groupId;
   }

   public double getTag() {
      return tag;
   }

   public void setTag(double tag) {
      this.tag = tag;
   }

   public int getTypeId() {
      return typeId;
   }

   public void setTypeId(int typeId) {
      this.typeId = typeId;
   }

   @Override
   public String toString() {
      final StringBuilder sb = new StringBuilder(super.toString());
      sb.append("SourceId:      ").append(getSourceId()).append("\n");
      sb.append("DestinationId: ").append(getDestinationId()).append("\n");
      sb.append("GroupId:       ").append(getGroupId()).append("\n");
      sb.append("TypeId:        ").append(getTypeId()).append("\n");
      sb.append("Tag:           ").append(getTag()).append("\n");
      return sb.toString();
   }
}
