/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.messages;

import ikr.simlib.events.calendar.Calendar;
import ikr.simlib.ports.Port;
import ikr.simlib.ports.PortMonitor;

import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * <code>MessageTracer</code> is a special port monitor that can be attached to
 * any port and will print all messages that are transferred thru that port. It
 * is a nice debugging aid...
 */
public class MessageTracer implements PortMonitor {

   private final PrintWriter log;

   public MessageTracer(PrintWriter log) {
      this.log = log;
   }

   public MessageTracer(PrintStream log) {
      this.log = new PrintWriter(log);
   }

   public MessageTracer(String logFile) throws FileNotFoundException {
      log = new PrintWriter(logFile);
   }

   @Override
   public void monitorGetMessage(Message message, Port port) {
      logMessage(log, message, port);
   }

   @Override
   public void monitorIsMessageAvailable(boolean messageAvailable, Port port) {
   }

   @Override
   public void monitorMessageIndication(Message message, Port port) {
   }

   /**
    * Prints the message to the log
    */
   public void logMessage(PrintWriter _log, Message msg, Port port) {
      _log.print("Message received at ");
      _log.print(port.getName());
      _log.print(" at ");
      _log.print(Calendar.getInstance().getSystemTime());
      _log.print(": ");
      _log.println(msg);
      _log.flush();
   }
}
