/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.messages.factories;

import ikr.simlib.factories.Factory;
import ikr.simlib.messages.IPPacket;
import ikr.simlib.messages.Message;
import ikr.simlib.model.SimNode;
import ikr.simlib.parameters.Parameters;
import ikr.simlib.parameters.reflection.ReflectionConstructable;
import ikr.simlib.parameters.reflection.ReflectionConstructable2;

public class IPPacketFactory implements Factory<Message>, ReflectionConstructable2<SimNode, Parameters> {

   private static final String LENGTH = "Length";
   private static final String SOURCE_ID = "SourceId";
   private static final String DESTINATION_ID = "DestinationId";
   private static final String GROUP_ID = "GroupId";
   private static final String TYPE_ID = "TypeId";
   private static final String TAG = "Tag";

   /** as required by {@link ReflectionConstructable} */
   public static IPPacketFactory createInstance(SimNode ownNode, Parameters pars) {
      long length = Message.DEFAULT_MESSAGE_LENGTH;
      if (pars.hasParameter(ownNode, LENGTH)) {
         length = pars.get(ownNode, LENGTH).asLong();
      }
      final int sourceId = pars.get(ownNode, SOURCE_ID).asInteger();
      final int destinationId = pars.get(ownNode, DESTINATION_ID).asInteger();
      final int groupId = pars.get(ownNode, GROUP_ID).asInteger();
      final int typeId = pars.get(ownNode, TYPE_ID).asInteger();
      final double tag = pars.get(ownNode, TAG).asDouble();
      return new IPPacketFactory(length, sourceId, destinationId, groupId, typeId, tag);
   }

   public final long length;
   public final int sourceId;
   public final int destinationId;
   public final int groupId;
   public final int typeId;
   public final double tag;

   public IPPacketFactory(long length, int sourceId, int destinationId, int groupId, int typeId, double tag) {
      this.length = length;
      this.sourceId = sourceId;
      this.destinationId = destinationId;
      this.groupId = groupId;
      this.typeId = typeId;
      this.tag = tag;
   }

   @Override
   public IPPacket create() {
      IPPacket packet = new IPPacket(length, sourceId, destinationId, groupId, typeId);
      packet.setTag(tag);
      return packet;
   }
}
