/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.meters.rate;

import ikr.simlib.events.time.Duration;
import ikr.simlib.meters.length.MessageLengthMeter;
import ikr.simlib.statistics.UnweightedSampleCollector;
import ikr.simlib.statistics.aggregating.IntervalAggregatingStatistic;
import ikr.simlib.statistics.aggregating.RateAggregator;

public abstract class IntervalRateMeter {
   /**
    * Helper method to create something which resembles the
    * <code>ShortTermRateMeter</code>, which was included in previous versions
    * of this library. Combines a {@link MessageLengthMeter} with an
    * {@link IntervalAggregatingStatistic}, so that the average rate over an
    * interval is calculated. Unit of measurement is message length units per
    * second.
    * <p>
    * The returned meter can be connected to one or to multiple ports. If it is
    * connected to multiple ports, the sum rate over all these ports is
    * calculated and written to the statistic.
    * <p>
    * To resemble the behaviour of the <code>ShortTermRateMeter</code> with the
    * parameter <code>measurePortsIndividually</code> set to true, create
    * multiple of these meters which are backed by a single instance of a
    * statistic.
    * 
    * @param interval
    *           duration of the averaging interval
    * @param statistic
    *           the statistic which is updated with the aggregated values
    * @return a new meter instance
    */
   public static MessageLengthMeter create(Duration interval, UnweightedSampleCollector<Double> statistic) {
      return MessageLengthMeter.createWithLongStatistic(new IntervalAggregatingStatistic<Long, Double>(statistic, interval, new RateAggregator()));
   }
}
