/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.parameters.fileparser.input;

import ikr.simlib.parameters.ParameterException;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;

public class LineReader implements Iterator<Line> {

   private int lineNumber;
   private ArrayList<String> lines;

   public LineReader(InputStream input) {
      BufferedReader reader = new BufferedReader(new InputStreamReader(input));

      lines = new ArrayList<String>();

      // read all lines
      String line = null;
      try {
         line = reader.readLine();
         while (line != null) {
            lines.add(line);
            line = reader.readLine();
         }

      } catch (IOException e) {
         throw new ParameterException("error while reading file: " + e.getMessage());
      }

      lineNumber = 0;
   }

   @Override
   public boolean hasNext() {
      return lines.size() > lineNumber;
   }

   @Override
   public Line next() {
      lineNumber++;
      return new Line(lineNumber, lines.get(lineNumber - 1));
   }

   @Override
   public void remove() {
      throw new ParameterException("Removing is not supported.");
   }

}
