/*
 * Copyright (C) 2007-2014 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

/**
 * <h1>Simulation Parameters</h1> <br>
 * The IKR Simulation Library can read simulation parameters from a file, the
 * so-called parameter file. How to write a parameter file and how to read
 * parameters inside the simulation program will be explained in the following.<br>
 * <br>
 * <h2>Parameter File Structure</h2> The parameter file will be read on a
 * per-line basis. Each line must have the following structure:<br>
 * <br>
 * "key" = "value"<br>
 * <br>
 * There are two principle for defining keys:<br>
 * <br>
 * - The key may be any name, which the simulation program can query for.
 * Organizing the name space must be done by the programmer.<br>
 * <br>
 * - The key or the first part of the key is the fully qualified name of the
 * SimNode that shall read the corresponding parameter(s). Names may contain
 * wildcards to make the key match different queries.<br>
 * <br>
 * The value will be interpreted as follows:
 * <ul>
 * <li>Whitespaces will be interpreted as separators of vectors. Scalar values
 * containing a whitespace character must be put in quotes.
 * <li>Vectors must begin with a [ character and end with a ] character. Inside a
 * vector, the values must be separated with whitespaces.
 * <li>A Vector can again contain vectors.
 * </ul>
 * You can use the # character to write comments. All characters of a line
 * following the # character will be ignored. Empty lines will be ignored.<br>
 * <br>
 * Example parameter file:<br>
 * # Calls = 10000<br>
 * TandemModel.*.NrOrGenerator = 42 # the answer<br>
 * TandemModel.Node*.Generator = StdGenerator<br>
 * TandemModel.Node*.Generator*.IATDist = NegExp<br>
 * TandemModel.Node*.Generator*.IATDist.Mean = 50<br>
 * <br>
 * <h2>Reading Parameter Values</h2> Class Parameters<br>
 * In every SimLib simulation program, there is one object of class
 * {@link ikr.simlib.parameters.Parameters}. This object represents the content
 * of the parameter file. The object provides a set of methods to query the
 * parameters listed in this file. The methods, that can be used for querying,
 * are the following ones:
 * <ul>
 * <li><code>hasParameter(<query>)</code> returns true if the parameter exists.
 * <li><code>hasSubTree()</code> returns true if the subtree exists.
 * <li><code>get(<query>)</code> returns the value of the parameter.
 * <li><code>getOrUseDefault(<query>, <default value>)</code> returns the value of
 * the parameter if it is listed in the parameter file. If not, the default
 * value will be used. Note that the default value will be passed as a string
 * and will only be parsed if it is needed.
 * </ul>
 * There are different variants for providing the query to the methods listed
 * above:
 * <ul>
 * <li>(String name): The name must specify the fully qualified name of the
 * queried parameter.
 * <li>(Prefix prefix, String name): The name will be extended by the prefix.
 * <li>(SimNode simNode, String name): The name will be extended by the name of
 * the simNode.
 * </ul>
 * <h2>Class Value</h2> The class {@link ikr.simlib.parameters.values.Value} is
 * the abstract base class of the classes
 * {@link ikr.simlib.parameters.values.Scalar} and
 * {@link ikr.simlib.parameters.values.Array}. These objects provide methods to
 * return a parameter value in one of the following forms:
 * <ul>
 * <li><code>boolean</code>
 * <li><code>enum</code>
 * <li><code>String</code>, <code>String[]</code>, or <code>String[][]</code>
 * <li><code>double</code>, <code>double[]</code>, or <code>double[][]</code>
 * <li><code>int</code>, <code>int[]</code>, or <code>int[][]</code>
 * <li><code>long</code>, <code>long[]</code>, or <code>long[][]</code>
 * </ul>
 * <h2>Parser Classes</h2> With the help of the <code>createInstance()</code>
 * method it is possible to create objects from classed defined in the parameter
 * file.
 */
package ikr.simlib.parameters;