/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.parameters.values;

import ikr.simlib.parameters.fileparser.FileParser;
import ikr.simlib.parameters.fileparser.tokens.Token;
import ikr.simlib.parameters.fileparser.tokens.TokenArrayIterator;
import ikr.simlib.parameters.fileparser.tokens.TokenExpander;
import ikr.simlib.parameters.reflection.ReflectionConstructable;
import ikr.simlib.parameters.reflection.ReflectionConstructable1;
import ikr.simlib.parameters.reflection.ReflectionConstructable10;
import ikr.simlib.parameters.reflection.ReflectionConstructable2;
import ikr.simlib.parameters.reflection.ReflectionConstructable3;
import ikr.simlib.parameters.reflection.ReflectionConstructable4;
import ikr.simlib.parameters.reflection.ReflectionConstructable5;
import ikr.simlib.parameters.reflection.ReflectionConstructable6;
import ikr.simlib.parameters.reflection.ReflectionConstructable7;
import ikr.simlib.parameters.reflection.ReflectionConstructable8;
import ikr.simlib.parameters.reflection.ReflectionConstructable9;

import java.util.List;

public abstract class Value {

   public abstract boolean asBoolean();

   public abstract <T extends Enum<T>> T asEnum(Class<T> enumClass);

   public abstract String asString();

   public abstract String[] asStringVector();

   public abstract String[][] asStringMatrix();

   public abstract double asDouble();

   public abstract double[] asDoubleVector();

   public abstract double[][] asDoubleMatrix();

   public abstract int asInteger();

   public abstract int[] asIntegerVector();

   public abstract int[][] asIntegerMatrix();

   public abstract long asLong();

   public abstract long[] asLongVector();

   public abstract long[][] asLongMatrix();

   public static Value fromString(String defaultValue) {
      List<Token> tokenList = TokenExpander.getParts(defaultValue, FileParser.OPENING_ARRAY_BRACKET, FileParser.CLOSING_ARRAY_BRACKET, FileParser.COMMENT);
      return FileParser.expand(null, new TokenArrayIterator(tokenList));
   }

   public abstract <P0, P1, P2, P3, P4, P5, P6, P7, P8, P9, T extends ReflectionConstructable10<P0, P1, P2, P3, P4, P5, P6, P7, P8, P9>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5, P6 p6, P7 p7, P8 p8, P9 p9);

   public abstract <P0, P1, P2, P3, P4, P5, P6, P7, P8, T extends ReflectionConstructable9<P0, P1, P2, P3, P4, P5, P6, P7, P8>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5, P6 p6, P7 p7, P8 p8);

   public abstract <P0, P1, P2, P3, P4, P5, P6, P7, T extends ReflectionConstructable8<P0, P1, P2, P3, P4, P5, P6, P7>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5, P6 p6, P7 p7);

   public abstract <P0, P1, P2, P3, P4, P5, P6, T extends ReflectionConstructable7<P0, P1, P2, P3, P4, P5, P6>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5, P6 p6);

   public abstract <P0, P1, P2, P3, P4, P5, T extends ReflectionConstructable6<P0, P1, P2, P3, P4, P5>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5);

   public abstract <P0, P1, P2, P3, P4, T extends ReflectionConstructable5<P0, P1, P2, P3, P4>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4);

   public abstract <P0, P1, P2, P3, T extends ReflectionConstructable4<P0, P1, P2, P3>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3);

   public abstract <P0, P1, P2, T extends ReflectionConstructable3<P0, P1, P2>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2);

   public abstract <P0, P1, T extends ReflectionConstructable2<P0, P1>> T asInstance(Class<T> clazz, P0 p0, P1 p1);

   public abstract <P0, T extends ReflectionConstructable1<P0>> T asInstance(Class<T> clazz, P0 p0);

   public abstract <T extends ReflectionConstructable> T asInstance(Class<T> clazz);

}
