/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.ports.input;

import ikr.simlib.entities.Entity;
import ikr.simlib.messages.Message;

/**
 * PolledInputPort is a special input port that is polled by the application,
 * e.g., IsMessageAvailable is called. Therefore, MessageIndication is ignored.
 * 
 * <p>
 * Message will be polled, HandleMessageIndication is ignored
 * </p>
 */
public class PolledInputPort extends InputPort {

   public PolledInputPort(Entity owner, String portName) {
      super(owner, portName);
   }

   public PolledInputPort(Entity owner) {
      this(owner, INPUT_PORT_NAME);
   }

   @Override
   protected void handleMessageIndication(Message msg) {
      // just ignore...
   }
}
