/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.ports.output;

import ikr.simlib.entities.Entity;
import ikr.simlib.messages.Message;

/**
 * Convenience class to use instead of manually deriving from {@link OutputPort}
 * .
 * <p>
 * Allows to specify two callback functions which are executed whenever the
 * connected input port queries whether a message is available or fetches the
 * message, respectively. The simple callback interfaces are intended to make
 * usage with java 1.8 closure syntax more convenient.
 */
public class CallbackOutputPort extends OutputPort {

   private final GetMessagehandler getMessageHandler;
   private final IsMessageAvailableHandler isMessageAvailableHandler;

   public interface GetMessagehandler {
      Message handleGetMessage();
   }

   public interface IsMessageAvailableHandler {
      boolean handleIsMessageAvailable();
   }

   public CallbackOutputPort(Entity owner, String portName, GetMessagehandler getMessageHandler, IsMessageAvailableHandler isMessageAvailableHandler) {
      super(owner, portName);
      this.getMessageHandler = getMessageHandler;
      this.isMessageAvailableHandler = isMessageAvailableHandler;
   }

   public CallbackOutputPort(Entity owner, GetMessagehandler getMessageHandler, IsMessageAvailableHandler isMessageAvailableHandler) {
      this(owner, OutputPort.OUTPUT_PORT_NAME, getMessageHandler, isMessageAvailableHandler);
   }

   @Override
   protected Message handleGetMessage() {
      return getMessageHandler.handleGetMessage();
   }

   @Override
   protected boolean handleIsMessageAvailable() {
      return isMessageAvailableHandler.handleIsMessageAvailable();
   }

}
