/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.ports.output;

import ikr.simlib.entities.Entity;
import ikr.simlib.messages.Message;
import ikr.simlib.ports.Port;
import ikr.simlib.ports.PortException;
import ikr.simlib.ports.input.InputPort;

public abstract class OutputPort extends Port {

   public static final String OUTPUT_PORT_NAME = "output";

   protected InputPort next;

   public OutputPort(Entity owner, String portName) {
      super(owner, portName);
   }

   public OutputPort(Entity owner) {
      this(owner, OUTPUT_PORT_NAME);
   }

   @Override
   public Port getConnectedPort() {
      return next;
   }

   public InputPort getConnectedInputPort() {
      return next;
   }

   public void setConnectedInputPort(InputPort inputPort) {
      next = inputPort;
   }

   @Override
   protected void handleMessageIndication(Message msg) {
      if (next != null) {
         // call connected input port
         next.messageIndication(msg);

      } else {
         // nobody handled it, throw port exception
         throw new PortException("MessageIndication() was not handled at port: " + getName() + ".");
      }
   }

   public void connect(InputPort inputPort) {
      // get owner entities
      Entity fromEntity = getOwner();
      Entity toEntity = inputPort.getOwner();

      // check if ports are already in use
      if (getConnectedInputPort() != null) {
         throw new PortException("Port already in use: " + getName());
      }

      if (inputPort.getConnectedOutputPort() != null) {
         throw new PortException("Port already in use: " + inputPort.getName());
      }

      LOG.fine("connecting port " + getName() + " to port " + inputPort.getName());

      // this is a valid connection
      setConnectedInputPort(inputPort);
      inputPort.setConnectedOutputPort(this);
   }

   public void disconnect(InputPort toPort) {
      getConnectedInputPort().setConnectedOutputPort(null);
      setConnectedInputPort(null);
   }
}
