/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.random;

/**
 * RandomNumberGenerator is the abstract base class for all random number
 * generators. New generators just have to override next() to provide new
 * numbers for each call.
 * 
 * <p>
 * Usually only one RNG object is used in a simulation. Therefore a global RNG
 * is created that can be accessed by the static member function
 * <code>getSystemRNG()</code> of RandomNumberGenerator.
 * </p>
 * 
 * <p>
 * Several standard RNGs are provided.
 * </p>
 */
public abstract class RandomNumberGenerator {

   protected static RandomNumberGenerator SYSTEM_RNG = null;

   public static RandomNumberGenerator getSystemRNG() {
      return SYSTEM_RNG;
   }

   public static void setSystemRNG(RandomNumberGenerator rng) {
      SYSTEM_RNG = rng;
   }

   /**
    * @return a new random value.
    */
   public abstract double next();
}
