/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.result;

import ikr.simlib.result.exceptions.PrintException;

/**
 * An extended result writer which supports print keywords (and thus formatting)
 * to write results.
 * 
 */
public interface PrintResultWriter extends ResultWriter {

   /**
    * Writes an long integer result for the given keyword.
    * 
    * @param keyword
    *           the print keyword which is used for the name and formatting
    * @param value
    *           the result value
    */
   void writeLong(ResultKeyword keyword, long value) throws PrintException;

   /**
    * Writes an double result for the given keyword.
    * 
    * @param keyword
    *           the print keyword which is used for the name and formatting
    * @param value
    *           the result value
    */
   void writeDouble(ResultKeyword keyword, double value) throws PrintException;

   /**
    * Writes an string result for the given keyword.
    * 
    * @param keyword
    *           the print keyword which is used for the name and formatting
    * @param value
    *           the result value
    */
   void writeString(ResultKeyword keyword, String value) throws PrintException;

}
