/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.result.filtering;

import ikr.simlib.result.PathHelper;
import ikr.simlib.result.PrintResultWriter;
import ikr.simlib.result.ResultKeyword;
import ikr.simlib.result.ResultWriter;
import ikr.simlib.result.exceptions.PrintException;

public class PrintResultFilterWriter implements PrintResultWriter {

   public static final String PATH_DELIMITER = new String(":");

   private final ResultWriter writer;
   private final PathHelper pathHelper;
   private final Filter filter;

   public PrintResultFilterWriter(ResultWriter writer) {
      if (writer == null) {
         throw new NullPointerException();
      }

      this.writer = writer;
      this.pathHelper = new PathHelper();
      this.filter = Filter.getInstance();
   }

   @Override
   public void close() throws PrintException {
      writer.close();
   }

   @Override
   public void writeDouble(ResultKeyword keyword, double value) throws PrintException {
      writeDouble(keyword.toString(), value);
   }

   @Override
   public void writeLong(ResultKeyword keyword, long value) throws PrintException {
      writeLong(keyword.toString(), value);
   }

   @Override
   public void writeString(ResultKeyword keyword, String value) throws PrintException {
      writeString(keyword.toString(), value);
   }

   @Override
   public void writeDouble(String name, double value) throws PrintException {
      if (printValue(name)) {
         writer.writeDouble(name, value);
      }
   }

   @Override
   public void writeLong(String name, long value) throws PrintException {
      if (printValue(name)) {
         writer.writeLong(name, value);
      }
   }

   @Override
   public void writeString(String name, String value) throws PrintException {
      if (printValue(name)) {
         writer.writeString(name, value);
      }
   }

   @Override
   public void writeStartArray(String name) throws PrintException {
      writer.writeStartArray(name);
      pathHelper.enterSubTree(name);
   }

   @Override
   public void writeEndArray() throws PrintException {
      writer.writeEndArray();
      pathHelper.leaveSubTree();
   }

   @Override
   public void writeStartArrayItem(long index) throws PrintException {
      writer.writeStartArrayItem(index);
      pathHelper.enterSubTree(Long.toString(index));
   }

   @Override
   public void writeEndArrayItem() throws PrintException {
      writer.writeEndArrayItem();
      pathHelper.leaveSubTree();
   }

   @Override
   public void writeStartNode(String type, String name) throws PrintException {
      writer.writeStartNode(type, name);
      pathHelper.enterSubTree(name);
   }

   @Override
   public void writeEndNode() throws PrintException {
      writer.writeEndNode();
      pathHelper.leaveSubTree();
   }

   @Override
   public void writeStartResults() throws PrintException {
      writer.writeStartResults();
   }

   @Override
   public void writeEndResults() throws PrintException {
      writer.writeEndResults();
   }

   private boolean printValue(String name) {
      String pathName = pathHelper.getPathName(name);
      return filter.printResult(pathName);
   }
}
