/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.statistics.distribution.trace.data;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class ListDataSource implements DataSource {
   private final List<Sample> list;

   public ListDataSource(List<Sample> list) {
      this.list = list;
   }

   public List<Sample> getList() {
      return list;
   }

   @Override
   public Iterator<Sample> iterator() {
      return list.iterator();
   }

   @Override
   public DataSource combineSorted(List<DataSource> input) {
      long totalSamples = 0;
      for (DataSource batchList : input)
         totalSamples += ((ListDataSource) batchList).list.size();
      if (totalSamples >= Integer.MAX_VALUE)
         throw new RuntimeException("too many values: " + totalSamples);
      final Sample[] allData = new Sample[(int) totalSamples];
      int idx = 0;
      for (DataSource batchList : input) {
         final List<Sample> bl = ((ListDataSource) batchList).list;
         System.arraycopy(bl.toArray(new Sample[0]), 0, allData, idx, bl.size());
         idx += bl.size();
      }
      Arrays.sort(allData);
      return new ListDataSource(Arrays.asList(allData));
   }
}
