/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.statistics.distribution.trace.data;

import ikr.simlib.model.SimNode;

import java.io.File;

/**
 * {@link DataStorage} backed by a file.
 */
public class TraceDataStorage implements DataStorage {
   private final TraceWriter writer;
   private boolean closed = false;

   public TraceDataStorage(SimNode namingNode, int batchNumber, boolean mergeAdjacentSamples) {
      this.writer = new TraceWriter(namingNode, batchNumber, mergeAdjacentSamples);
   }

   public File getFile() {
      return writer.getFile();
   }

   @Override
   public void write(Sample sample) {
      assert !closed;
      writer.write(sample);
   }

   @Override
   public void sortInPlace() {
      assert closed;
      try {
         TraceSorter.sort(writer.getFile());
      } catch (TraceFilesInvalidException e) {
         writer.getFile().delete();
      }
   }

   @Override
   public void writingCompleted() {
      writer.close();
      closed = true;
   }

   @Override
   public DataSource getDataSource() {
      assert closed;
      return new TraceDataSource(writer.getFile());
   }

}
