/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.statistics.distribution.trace.data;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Iterator;
import java.util.zip.Inflater;
import java.util.zip.InflaterInputStream;

public class TraceReader {
   private static final int BUFFER_SIZE = 102400; // in byte

   private final DataInputStream stream;
   private final Iterator<Sample> iter;
   private boolean isClosed = false;

   public TraceReader(File file) throws TraceFilesInvalidException {
      try {
         stream = new DataInputStream(new BufferedInputStream(new InflaterInputStream(new BufferedInputStream(new FileInputStream(file), BUFFER_SIZE), new Inflater(), BUFFER_SIZE), BUFFER_SIZE));
      } catch (FileNotFoundException e) {
         throw new TraceFilesInvalidException();
      }
      iter = new Iterator<Sample>() {
         Sample next = null;

         @Override
         public boolean hasNext() {
            if (next == null && !isClosed)
               try {
                  next = readNext();
               } catch (EOFException e) {
                  next = null;
               }
            final boolean hasNext = next != null;
            if (!hasNext)
               close();
            return hasNext;
         }

         @Override
         public Sample next() {
            hasNext();
            final Sample s = next;
            next = null;
            return s;
         }

         @Override
         public void remove() {
            throw new UnsupportedOperationException();
         }
      };
   }

   public Iterator<Sample> getIterator() {
      return iter;
   }

   public void close() {
      isClosed = true;
      try {
         stream.close();
      } catch (IOException e) {
         throw new RuntimeException(e);
      }
   }

   public Sample readNext() throws EOFException {
      try {
         return new Sample(stream.readDouble(), stream.readDouble());
      } catch (EOFException eof) {
         throw eof;
      } catch (IOException e) {
         throw new RuntimeException(e);
      }
   }
}
