/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.statistics.distribution.trace.dist;

import ikr.simlib.statistics.distribution.trace.quantile.QuantileDescription;

import java.util.List;

public class MinMaxDistributionData extends DistributionData {

   public MinMaxDistributionData(QuantileDescription quantiles, List<DropDistributionData> drops, boolean minimum) {
      super(quantiles);
      // calculate minimum or maximum
      for (int quantile = 0; quantile < getNumberOfQuantiles(); quantile++) {
         double val = minimum ? Double.POSITIVE_INFINITY : Double.NEGATIVE_INFINITY;
         for (DropDistributionData drop : drops) {
            assert drop.getNumberOfQuantiles() == getNumberOfQuantiles();
            if (minimum)
               val = Math.min(val, drop.getQuantileValue(quantile));
            else
               val = Math.max(val, drop.getQuantileValue(quantile));
         }
         quantileValues[quantile] = val;
      }
      smallestValue = minimum ? Double.POSITIVE_INFINITY : Double.NEGATIVE_INFINITY;
      for (DropDistributionData drop : drops)
         if (minimum)
            smallestValue = Math.min(smallestValue, drop.getSmallestValue());
         else
            smallestValue = Math.max(smallestValue, drop.getSmallestValue());
   }

}