/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.statistics.distribution.trace.quantile;

/**
 * Generates quantiles to be used in a CCDF with logarithmic y axis. Quantiles
 * start from zero and asymptotically approach 1. A quantile value for 1 will
 * not be generated.
 */
public class InverseLogQuantiles implements QuantileDescription {
   private final LogarithmicQuantiles logQuantiles;

   public InverseLogQuantiles(int numberOfQuantiles, double largestValue) {
      this.logQuantiles = new LogarithmicQuantiles(numberOfQuantiles, 1 - largestValue);
   }

   @Override
   public int getNumberOfQuantiles() {
      return logQuantiles.getNumberOfQuantiles();
   }

   @Override
   public double getRelativeQuantilePosition(int quantileIndex) {
      assert quantileIndex < getNumberOfQuantiles() && quantileIndex >= 0;
      final int inverseIndex = getNumberOfQuantiles() - 1 - quantileIndex;
      final double logQuantile = logQuantiles.getRelativeQuantilePosition(inverseIndex);
      return 1d - logQuantile;
   }

   public static void main(String[] args) {
      InverseLogQuantiles test = new InverseLogQuantiles(51, 0.99999);
      for (int i = 0; i < test.getNumberOfQuantiles(); i++)
         System.out.println(test.getRelativeQuantilePosition(i));
   }
}
