/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.statistics.support;

import ikr.simlib.exceptions.SimulationException;

public class LogBucketUtility extends BucketUtility {

   public static double DEFAULT_MEAN = 0;

   private final double logMin;
   private final double logMax;

   public LogBucketUtility(double min, double max, int noOfBuckets) {
      super(min, max, noOfBuckets);

      logMin = Math.log(min);
      logMax = Math.log(max);

      if (Double.isInfinite(logMin) || Double.isNaN(logMin))
         throw new RuntimeException("invalid minimum value " + min);
      if (Double.isInfinite(logMax) || Double.isNaN(logMax))
         throw new RuntimeException("invalid maximum value " + max);
   }

   @Override
   public int getBucket(double condVal) {
      if (!isInRange(condVal)) {
         throw new SimulationException("condition value out of range");
      }

      int bucket = (int) Math.floor(getArraySize() * (Math.log(condVal) - logMin) / (logMax - logMin));

      assert bucket <= getArraySize();
      if (bucket == getArraySize()) { // numerical problem
         bucket = getArraySize() - 1;
      }

      return bucket;
   }

   @Override
   public double getLowerBound(int index) {
      if (index >= getArraySize()) {
         throw new SimulationException("index out of range");
      }

      return Math.exp(((double) index / getArraySize()) * (logMax - logMin) + logMin);
   }

   @Override
   public double getUpperBound(int index) {
      if (index >= getArraySize()) {
         throw new SimulationException("index out of range");
      }

      return Math.exp(((double) (index + 1) / getArraySize()) * (logMax - logMin) + logMin);
   }

}
