/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.support;

import java.io.IOException;
import java.io.OutputStream;

/**
 * Wraps an output stream. The opening of the wrapped stream is delayed until
 * the first byte is written. For new files, this also delays the file creation.
 */
public abstract class DelayedOutputStream extends OutputStream {

   private boolean opened;
   private OutputStream underlying;

   private void ensureOpened() {
      if (!opened) {
         opened = true;
         underlying = openUnderlying();
      }
   }

   protected abstract OutputStream openUnderlying();

   @Override
   public void write(int b) throws IOException {
      ensureOpened();
      underlying.write(b);
   }

   @Override
   public void write(byte[] b, int off, int len) throws IOException {
      ensureOpened();
      underlying.write(b, off, len);
   }

   @Override
   public void write(byte[] b) throws IOException {
      ensureOpened();
      underlying.write(b);
   }

   @Override
   public void flush() throws IOException {
      ensureOpened();
      underlying.flush();
   }

   @Override
   public void close() throws IOException {
      ensureOpened();
      underlying.close();
   }

}
